/*
 * Demo proc entry
 *
 * Written by Tilmann Bitterberg
 */

#include <linux/module.h>
#include <linux/init.h> /* for module_init */
#include <linux/errno.h>
#include <linux/sched.h> /* for task struct */
#include <linux/proc_fs.h>

#include <asm/uaccess.h>

static ssize_t proc_demo_read(struct file * file, char * buf,
			     size_t count, loff_t *ppos);
static ssize_t proc_demo_write(struct file * file, const char * buffer,
			      size_t count, loff_t *ppos);
struct file_operations proc_demo_operations = {
	owner:	        THIS_MODULE,
        read:           proc_demo_read,
        write:          proc_demo_write,
};

	

static ssize_t proc_demo_read(struct file * file, char * buf,
			     size_t count, loff_t *ppos)
{
	int n = 0;
	struct task_struct *p;
	char buffer[512];

	static int number=0;

	number++;

	n  = sprintf (buffer  , 
	      "Ich wurde %dmal gelesen\nPids aktive: \n", number);
	for_each_task(p)
	     n += sprintf (buffer+n, "%d ", p->pid);
	n += sprintf (buffer+n, "\n");
			
	if (count < -1)
		return -EINVAL;

	if (*ppos >= strlen(buffer))
		return 0;
	if (n > strlen(buffer) - *ppos)
		n = strlen(buffer) - *ppos;
	if (n > count)
		n = count;
	copy_to_user(buf, buffer + *ppos, n);
	*ppos += n;
	return n;
}

static ssize_t proc_demo_write(struct file * file, const char * buffer,
			      size_t count, loff_t *ppos)
{
	return 0;
}

MODULE_DESCRIPTION("Proc demo module");
MODULE_AUTHOR("Tilmann Bitterberg");

static void cleanup_demoproc(void)
{
	remove_proc_entry("hft-demo", NULL);
}

static int init_demoproc(void)
{
	struct proc_dir_entry *entry;
	entry = create_proc_entry("hft-demo", S_IRUGO|S_IWUSR, NULL);
	if (entry) entry->proc_fops = &proc_demo_operations;
   	
	return 0;
}

module_init(init_demoproc);
module_exit(cleanup_demoproc);
